'use strict';

var uuid = require('uuid');
var _ = require('lodash');
var objectHash = require('object-hash');

var _require = require('electron');

var ipcRenderer = _require.ipcRenderer;

function run(moduleHash, funcName, args) {
  var eventKey = uuid.v4();
  var payload = {
    moduleHash: moduleHash,
    funcName: funcName,
    args: args,
    eventKey: eventKey
  };

  return new Promise(function (resolve, reject) {
    function taskCompleteCallback(data) {
      var resultType = data.resultType;
      var result = data.result;
      var reason = data.reason;
      var replyEventKey = data.eventKey;

      if (replyEventKey === eventKey) {
        switch (resultType) {
          case 'BACKGROUND_RESOLVE':
            ipcRenderer.removeListener('BACKGROUND_REPLY', taskCompleteCallback);
            resolve(result);
            break;
          case 'BACKGROUND_REJECT':
            ipcRenderer.removeListener('BACKGROUND_REPLY', taskCompleteCallback);
            reject(reason);
            break;
          default:
        }
      }
    }
    ipcRenderer.on('BACKGROUND_REPLY', taskCompleteCallback);
    ipcRenderer.send('BACKGROUND_START', payload);
  });
}

var foreground = {
  getModule: function getModule(moduleName) {
    var promiseWrappedModule = {};
    var originalModule = require(moduleName);
    var moduleHash = objectHash(originalModule);
    _.forEach(originalModule, function (func, funcName) {
      if (_.isFunction(func)) {
        promiseWrappedModule[funcName] = function () {
          var args = _.map(arguments, function (element) {
            return element;
          });
          return run(moduleHash, funcName, args);
        };
      }
    });
    return promiseWrappedModule;
  }
};

module.exports = foreground;